@@ -3,7 +3,7 @@ package Params::CheckCompiler::Compiler;
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use Eval::Closure;
 use List::SomeUtils qw( first_index );
@@ -14,9 +14,20 @@ use Params::CheckCompiler::Exception::Positional::Extra;
 use Params::CheckCompiler::Exception::Positional::Required;
 use Params::CheckCompiler::Exception::ValidationFailedForMooseTypeConstraint;
 use Scalar::Util qw( blessed looks_like_number reftype );
+use Sub::Name qw( subname );
 
 use Moo;
 
+has name => (
+    is        => 'ro',
+    predicate => '_has_name',
+);
+
+has caller => (
+    is        => 'ro',
+    predicate => '_has_caller',
+);
+
 has params => (
     is       => 'ro',
     required => 1,
@@ -41,10 +52,18 @@ sub subref {
     my $self = shift;
 
     $self->_compile;
-    return eval_closure(
+    my $sub = eval_closure(
         source => 'sub { ' . ( join "\n", @{ $self->_source } ) . ' };',
         environment => $self->_env,
     );
+
+    if ( $self->_has_name ) {
+        my $caller = $self->_has_caller ? $self->caller : caller(1);
+        my $name = join '::', $caller, $self->name;
+        subname( $name, $sub );
+    }
+
+    return $sub;
 }
 
 sub source {
@@ -75,7 +94,7 @@ sub _compile_named_args_check {
 
     my $params = $self->params;
 
-    for my $name ( keys %{$params} ) {
+    for my $name ( sort keys %{$params} ) {
         my $spec = $params->{$name};
         $spec = { optional => !$spec } unless ref $spec;
 
@@ -569,7 +588,7 @@ Params::CheckCompiler::Compiler - Object that implements the check subroutine co
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =for Pod::Coverage .*
 
